## R codes of the article:

## Hitchhikers’ guide to analysing bird ringing data
## Part 1: data cleaning, preparation and exploratory analyses
## Harnos, A, Fehérvári, P. & Csörgő, T.
## Ornis Hungarica 2015. 23(2): 163-188.

## ----Load the ringR package------------------------------------
library(ringR)

## ----Set the working directory---------------------------------------
setwd("D:/mydirectory")

## ----Import data from a text file-------------------------------------
mydata = read.table(
   file = "FICHYP.csv", #the name of the file
   sep = ";",           #the field separator character
   dec = ".",           #the character used in the file for decimal points
   header = TRUE,  #a logical value indicating whether the file 
                   #contains the names of the variables in the first line
   na.string = c("null")) #a character vector of strings which are to be 
                          #interpreted as NA values

## ----Import data from an Excel table---------------------------------------
# library(xlsx)
# mydata = read.xlsx("FICHYP.xlsx", sheetIndex=1)

## ----Structure of the table----------------------------------------------
str(mydata)

## ----Convert numerically coded categorical variables to factors-------------------
mydata$FAT = factor(mydata$FAT)
str(mydata$FAT)  #check the structure
mydata$MUSCLE = factor(mydata$MUSCLE)
str(mydata$MUSCLE)  #check the structure
mydata$ID = factor(mydata$ID)
str(mydata$ID)  #check the structure

## -----------------View the table--------------------------------------
## ----Head (and tail) of the table----------------------------------------
head(mydata)
tail(mydata)

## ----Summary of the table------------------------------------------------
# Summary of the whole table
summary(mydata)
# Summary of AGE, SEX, FAT, WING and WING variables
summary(mydata[,c('AGE', 'SEX', 'FAT', 'MASS', 'WING')])

## ----Extract year and yearday from the DATE variable------------------
# The following symbols can be used in the format string:
# Symbol        Meaning                         Example
# %d            day as a number (0-31)          01-31
# %a            abbreviated weekday             Mon
# %A            unabbreviated weekday           Monday
# %m            month (00-12)                   00-12
# %b            abbreviated month               Jan
# %B            unabbreviated month             January
# %y            2-digit year                    07
# %Y            4-digit year                    2007

mydata = ring.date(
  data    = mydata,         #data frame
  datevar = 'DATE',         #date variable
  format  = "%m/%d/%Y")     #format of date in the data table
str(mydata)  # check the variables

## ----Determine first captures------------------------------------------
mydata = ring.firstcapture(
  data          =  mydata,   #data frame
  periodvar     = 'year',    #name of the period variable
  perioddayvar  = 'yearday', #name of the 'period day' variable
  IDvar         = 'RING')    #name of the ID variable

str(mydata)  #check the data table

## ----Plot capture and recapture frequencies by year----------------------
# Setting the graphical layout (see  ?par for details)
par(las=1, omi=c(0,0,0,0), mar=c(4, 4, 2, 2) + 0.1, bty='l')

plot.periodic.captures(
  data         = mydata,         #data frame
  periodvar    = 'year',         #name of the period variable
  perioddayvar = 'yearday',      #name of the 'period day' variable
  recapvar     = 'period.recap', #variable indicating captures (0) 
                                 #and recaptures (1)
  ydivider     = 20  )           #y axis divider, 0 means automatic setting

## ----Plot daily capture and recapture frequencies------------------------
plot.daily.captures(
  data         = mydata,           #data frame
  perioddayvar = 'yearday',        #name of the 'period day' variable
  recapvar     = 'period.recap',   #variable indicating captures (0) 
                                   #and recaptures (1)
  ylim = c(0,0), xlim = c(50,300), #limit of x and y axis
  xlab = "Day of the year",        #title for x axis
  ylab = "Frequency",              #title for y axis
  cap.color = "black",             #color of capture bars
  recap.color = "darkgray",        #color of recapture bars
  ydivider = 10)                   #y axis divider
                                    
## ----Plot cumulative captures--------------------------------------------
plot.cum.captures(
  data         = mydata,           #data frame
  IDvar        = 'RING',           #name of the ID variable
  periodvar    = 'year',           #name of the period variable
  perioddayvar = 'yearday')        #name of the 'period day' variable
 

## ----Create a season variable--------------------------------------------
#Separate observations according to migratory seasons with a cut-off point at the 150th day
mydata$season = ifelse(
  test = mydata$yearday < 150,   #logical condition
  yes  = 'spring',               #value if true
  no   = 'autumn')               #value if false
mydata$season = factor(mydata$season)  #conversion to factor

## ----Check the AGE variable--------------------------------------------
levels(mydata$AGE)     #factor levels
nlevels(mydata$AGE)    #number of levels
table(mydata$AGE)      #frequencies in categories defined by factor levels

## ----Recode the AGE variable-------------------------------------------
#Recode "1" to "1Y" 
mydata$AGE = ifelse(
  test = mydata$AGE == "1",         #logical condition
  yes  = "1Y",                      #value if true
  no   = as.character(mydata$AGE))  #value if false, as.character()  
                                    #needed to get character strings
mydata$AGE = factor(mydata$AGE)     #conversion to factor
levels(mydata$AGE)                  #check the levels of the variable

#Recode "2" to "2Y" 
mydata$AGE = ifelse(
  test = mydata$AGE == "2",         #logical condition
  yes  = "2Y",                      #value if true
  no   = as.character(mydata$AGE))  #value if false, as.character()  
                                    #needed to get character strings                                    
mydata$AGE = factor(mydata$AGE)     #convert to factor
levels(mydata$AGE)                  #check the levels of the variable

## ----Create the age variable-----------------------------------------------
#The new 'age' variable is "juv" if AGE is "1Y" and "adult", if AGE is 
#"1+" or "2Y" or "2+" otherwise NA
mydata$age = ifelse(
  test = mydata$AGE == "1Y",
  yes  = "juv", 
  no   = NA)

#The second ifelse() call takes the new age variable and recodes it so as 
#1+, 2+ and 2Y records become adult whereas all others remain either NA or juv.
mydata$age = ifelse(
  test = mydata$AGE == "1+" |
         mydata$AGE == "2Y" |
         mydata$AGE == "2+",
  yes  = "adult", 
  no   = mydata$age)

## ----Check and recode the SEX variable-------------------------------------------
levels(mydata$SEX)                  #levels
#recode "N" to NA
mydata$SEX = ifelse(
  test = mydata$SEX == "N" ,
  yes  = NA, 
  no   = as.character(mydata$SEX))  
mydata$SEX = factor(mydata$SEX)     #converr to factor

## ----Contingency tables--------------------------------------------------
#contingency table
table(mydata$age, mydata$SEX)
#relative frequencies according to row totals
prop.table(
  table(mydata$age, mydata$SEX),   #contingency table
  margin = 1)                      #indicating row totals
#relative frequencies according to coloumn totals
prop.table(
  table(mydata$age, mydata$SEX),   #contingency table
  margin = 2)                      #indicating column totals
#relative frequencies to grand totals
prop.table(
  table(mydata$age, mydata$SEX))   #contingency table

## ----Barplot of FAT scores-----------------------------------------------
barplot(
  table(mydata$FAT),               #frequency tables
  xlab = "Fat score",              #x axis title
  ylab = "Frequency")              #y axis title

## ----Mosaicplot of age and SEX------------------------------------------
with(mydata[mydata$RECAP == 0,],#subset of first captures  
  mosaicplot(table(age, SEX),   #mosaicplot call with the contingency table
    main = ""))                 #no title

## ----Range of MASS values-----------------------------------------------
range(mydata$MASS, na.rm = TRUE)   #na.rm=TRUE is needed

## ----Change 0-s to NA-s-----------------------------------------------
nrow(mydata[mydata$MASS == 0,])    #number of rows where MASS = 0
mydata$MASS = 
  ifelse(
    test = mydata$MASS == 0, 
    yes  = NA, 
    no   = mydata$MASS)  

## ----Cases where MASS > 20-----------------------------------------------
which(mydata$MASS > 20)       #Row numbers in mydata where MASS > 20
xx = which(mydata$MASS > 20)  #Creating xx vector containing row numbers 
                              #in mydata where MASS > 20
#List the cases
mydata[xx,]

## ----Change identified values of MASS variable-------------------------
mydata[mydata$ID == 
         244759,]$MASS = 15.5
mydata[mydata$ID == 
         355719,]$MASS = 13.2

## ----Change 0-s to NA in case of WING, THIRD and TAIL------------------
mydata$WING = ifelse(
  test = mydata$WING == 0, 
  yes  = NA, 
  no   = mydata$WING)
mydata$THIRD = ifelse(
  test = mydata$THIRD == 0, 
  yes  = NA, 
  no   = mydata$THIRD)
mydata$TAIL = ifelse(
  test = mydata$TAIL == 0,
  yes  = NA, 
  no   = mydata$TAIL)

## ----Subset of spring males-------------------------------------------
mydata.spring.M = mydata[       
  mydata$season == "spring" & 
    mydata$SEX == "M",]

## ----Create multiple Cleveland dotplots-------------------------------
par(mfrow = c(1,4))               #four graphs in a row
with(mydata.spring.M,             #Subset of spring males
  dotchart(MASS, xlab = "MASS"))
with(mydata.spring.M, 
  dotchart(WING, xlab = "WING"))
with(mydata.spring.M, 
  dotchart(THIRD, xlab = "THIRD"))
with(mydata.spring.M, 
  dotchart(TAIL, xlab = "TAIL"))
par(mfrow = c(1,1))               #one graph in a row

## ----Select rows where MASS > 18 in case of spring observations-------
xx = which(mydata[               #row numbers in mydata where MASS > 18
  mydata$season == "spring",]$MASS > 18)
#List the cases
mydata[mydata$season == "spring",][xx,]

## ----Select rows where TAIL > 60 in case of spring observations-------
xx = which(mydata[               #row numbers in mydata where MASS > 18
  mydata$season == "spring",]$TAIL > 60)
#List the cases
mydata[mydata$season == "spring",][xx,]

#Checked in the ringing notes, the long tails belong to relatively long wings,
#so no correction or deletion are needed

## ----Histograms----------------------------------------------------------
with(mydata[                         #subset of 
  mydata$RECAP  == 0 &               #first captures and
  mydata$age    == "juv" &           #juveniles and
  mydata$SEX    == "M" &             #males and
  mydata$season == "autumn",],       #in autumn
hist(MASS, main = "Males",           #histogram, main title = "Males"
  xlim = c(10,20), ylim  = c(0,140), #range of x and y axis
  breaks = 20, xlab = "Mass (g)"))   #number of breaks and x axis title

with(mydata[                         #subset of 
  mydata$RECAP  == 0 &               #first captures and
  mydata$age    == "juv" &           #juveniles and
  mydata$SEX    == "F" &             #females and
  mydata$season == "autumn",],       #in autumn
  hist(MASS, main = "",              #histogram, main title = "Females"
  xlim = c(10,20), ylim = c(0,140),  #range of x and y axis
  breaks = 20, xlab = "Mass (g)"))   #number of breaks and x axis title

## ----Smoothed histograms (density plots)--------------------------------
library(car)                         #loading the 'car' package
with(mydata[mydata$RECAP == 0 &      #subset of first captures and
    mydata$age == "juv" &            #juveniles 
    mydata$season == "autumn",],     #in autumn
densityPlot(MASS ~ SEX,              #density plots of MASs by SEX groups
  col  = c("black","darkgray"),      #colour of the two curves
  xlab = "Mass (g)"))                #title for x axis

## ----Boxplot of MASS-----------------------------------------------------
with(mydata[
  mydata$season == "autumn",],       #subset of autumn observations
  boxplot(MASS ~ age,                #boxplots of MASS by age groups 
    ylab = "Mass (g)",               #title for y axis
    names = c("adult", "juvenil")))  #group labels on the x axis

## ----List cases where MASS < 5----------------------------------------
which(mydata$MASS < 5)
mydata[which(mydata$MASS < 5),]

## ----Change MASS = 1.6 to MASS = 16------------------------------------
mydata[mydata$ID == 
         "583178",]$MASS = 16

## ----Boxplot of MASS in spring-------------------------------------------
with(mydata[
  mydata$season == "spring",],       #subset of autumn observations
  boxplot(MASS ~ SEX,                #boxplots of MASS by SEX groups 
          ylab = "Mass (g)",               #title for y axis
          names = c("adult", "juvenil")))  #group labels on the x axis

## ----Boxplot of MASS by age and SEX groups-------------------------------
with(mydata[mydata$season == "autumn",], #subset of autumn observations
  boxplot(MASS ~ age:SEX,         #boxplots of MASS by age and SEX groups 
    ylab     = "Mass (g)",        #title for y axis
    varwidth = TRUE))             #box widths are proportional to the 
                                  #square-roots of the number of observations 
                                  #in the groups.

## ----Check the WING variable ----------------------------------------
## ----Range of WING values-----------------------------------------------
range(mydata$WING, na.rm = TRUE)   #na.rm=TRUE is needed

## ----Boxplot of WING in spring by SEX groups----------------------------
with(mydata[
  mydata$season == "spring",],       #subset of autumn observations
  boxplot(WING ~ SEX,                #boxplots of WING by SEX groups 
          ylab = "WING (mm)"))       #title for y axis

## ----List cases where WING > 90----------------------------------------
which(mydata$WING > 90)
mydata[which(mydata$WING > 90),]

## ----Change WING = 91 to WING = NA------------------------------------
mydata[mydata$ID == 
         "531866",]$WING = NA
         
#recheck the boxplot
with(mydata[
  mydata$season == "spring",],       #subset of autumn observations
  boxplot(WING ~ SEX,                #boxplots of WING by SEX groups 
          ylab = "WING (mm)"))       #title for y axis

## ----Boxplot of WING by age and SEX groups in autumn--------------------
with(mydata[mydata$season == "autumn",], #subset of autumn observations
     boxplot(WING ~ age:SEX,         #boxplots of WING by age and SEX groups 
             ylab     = "WING (mm)", #title for y axis
             varwidth = TRUE))           

## ----Check the THIRD variable ----------------------------------------
## ----Range of THIRD values-----------------------------------------------
range(mydata$THIRD, na.rm = TRUE)   #na.rm=TRUE is needed


## ----Boxplot of THIRD in spring by SEX groups----------------------------
with(mydata[
  mydata$season == "spring",],       #subset of autumn observations
  boxplot(THIRD ~ SEX ,                #boxplots of THIRD by SEX groups 
          ylab = "THIRD (mm)"))      #title for y axis

## ----Boxplot of THIRD by age and SEX groups-------------------------------
with(mydata[mydata$season == "autumn",],   #subset of autumn observations
     boxplot(THIRD ~ age:SEX,         #boxplots of THIRD by age and SEX groups 
             ylab     = "THIRD (mm)", #title for y axis
             varwidth = TRUE))        #box widths are proportional to the 
                                      #square-roots of the number of 
                                      #observations in the groups.

## ----Check the TAIL variable ----------------------------------------
## ----Range of TAIL values-----------------------------------------------
range(mydata$TAIL, na.rm = TRUE)     #na.rm=TRUE is needed


## ----Boxplot of TAIL in spring by SEX groups----------------------------
with(mydata[
  mydata$season == "spring",],       #subset of autumn observations
  boxplot(TAIL ~ SEX ,                 #boxplots of TAIL by SEX groups 
          ylab = "THIRD (mm)"))      #title for y axis

## ----Boxplot of TAIL by age and SEX groups in autumn--------------------
with(mydata[mydata$season == "autumn",], #subset of autumn observations
     boxplot(TAIL ~ age:SEX,            #boxplots of TAIL by age and SEX groups 
             ylab     = "TAIL (mm)",    #title for y axis
             varwidth = TRUE))          #box widths are proportional to the 
                                        #square-roots of the number of  
                                        #observations in the groups.

## ----List cases where TAIL > 65----------------------------------------
which(mydata$TAIL > 65)
mydata[which(mydata$TAIL > 65),]

## ----Change TAIL = 72 to TAIL = NA------------------------------------
mydata[mydata$ID == 
         "646196",]$TAIL = NA

## Finally, check the numeric variables with Cleveland dotplots by subgroups
## ----Create multiple Cleveland dotplots for spring females-------------
mydata.spring.F = mydata[       
  mydata$season == "spring" & 
    mydata$SEX == "F",]
par(mfrow = c(1,4))               #four graphs in a row
with(mydata.spring.F,             #Subset of spring males
  dotchart(MASS, xlab = "MASS"))
with(mydata.spring.F, 
  dotchart(WING, xlab = "WING"))
with(mydata.spring.F, 
  dotchart(THIRD, xlab = "THIRD"))
with(mydata.spring.F, 
  dotchart(TAIL, xlab = "TAIL"))
par(mfrow = c(1,1))               #one graph in a row

## ----Create multiple Cleveland dotplots for autumn males------------
mydata.autumn.M = mydata[       
  mydata$season == "autumn" & 
    mydata$SEX == "M",]
par(mfrow = c(1,4))               #four graphs in a row
with(mydata.autumn.M,             #Subset of spring males
  dotchart(MASS, xlab = "MASS"))
with(mydata.autumn.M, 
  dotchart(WING, xlab = "WING"))
with(mydata.autumn.M, 
  dotchart(THIRD, xlab = "THIRD"))
with(mydata.autumn.M, 
  dotchart(TAIL, xlab = "TAIL"))
par(mfrow = c(1,1))               #one graph in a row

## ----Create multiple Cleveland dotplots for autumn females------------
mydata.autumn.F = mydata[       
  mydata$season == "autumn" & 
    mydata$SEX == "F",]
par(mfrow = c(1,4))               #four graphs in a row
with(mydata.autumn.F,             #Subset of spring males
  dotchart(MASS, xlab = "MASS"))
with(mydata.autumn.F, 
  dotchart(WING, xlab = "WING"))
with(mydata.autumn.F, 
  dotchart(THIRD, xlab = "THIRD"))
with(mydata.autumn.F, 
  dotchart(TAIL, xlab = "TAIL"))
par(mfrow = c(1,1))               #one graph in a row

#Note that there is a very small bird in the sample (lower part of the graphs) however, 
#other biometric measurements are also small of this individual.

#List all birds with wing length less than 75 mm:
mydata[which(mydata$WING<75),]

#All birds are juveniles

## ----Saving the data frame as a text file--------------------------------
write.table(mydata,                       #data frame to be saved
            file      = 'FICHYP1.csv',     #output file name
            sep       = ';',               #the field separator string in the output 
            dec       = '.',               #the string to use for decimal points
            quote     = FALSE,             #a logical value. If TRUE, any character 
                                           #or factor columns will be surrounded by 
                                           #double quotes in the output file
            row.names = FALSE)             #logical value indicating whether the 
                                           #row names of the data frame are to be written 
