## ----------------------------Introduction to R ---------------------------------

## --------------------------- R environment - Basics -----------------------------

## For installation instructions of R and RStudio see: 
## Paul Torfs & Claudia Brauer: A (very) short introduction to R: 
## https://cran.r-project.org/doc/contrib/Torfs+Brauer-Short-R-Intro.pdf

## For a more extended introductory tutorial, see:  
## https://cran.r-project.org/doc/manuals/r-release/R-intro.html

### General remarks for code (text) editing ###

## WORD WRAPPING
# allows breaking lines to be readable in an actual window independent of the window width. This utility is available for most text editors including the one RStudio uses for editing R-codes. 
# RStudio word wrapping for the R-source files (the scripts/codes you actually write & execute):
# 
# Tools > Global Options... > Code > Editing: Tick "Soft-wrap R source files" > Apply

## COMMENTING
## is used to insert user notes into the scripts/codes when programming. Many programming languages, including R, uses hashmarks '#' to define comments in the code. Any strings/texts following a hashmark to the end of the line is a comment an is NOT interpreted by R as commands. Selected lines can be easily commented/uncommented with text editors.
# RStudio: Ctrl Shift C

# More about text editors:
# https://docs.google.com/viewer?a=v&pid=sites&srcid=ZGVmYXVsdGRvbWFpbnxlZHVjYXRpb25qa3xneDoyYTUxMmVhYzViNThkN2Yy

## Submitting R commands
# After typing a command in the editor window in RStudio, click Run or press CTRL+ENTER to send it to the command window and to execute it.

## In case of using brackets and forgetting to add the closing bracket, the '>' promt of the command line changes to a '+'. If you want to get back the '>', press ESC. To avoid forgetting closing brackets, set RStudio to do it automatically when openeing one: Tools > Global Options... > Code > Editing: Insert matching parens/quotes . This applies to the following characters: (), {}, [], '', "".

## Suggestion! Go through the following examples, study the codes and the outputs, then try to write similar codes to practice coding!

### R-basics ###

# R can be used as a calculator. Just type your mathematical expression in the command window after the promt ('>'), or type it in the script window and run it:

1+1 # One plus One

## Help system
# A large amount of free documentation and help are available. 

# Look up the help page of a specific function
help(t.test)
?t.test
# It gives a description of the function, possible arguments and the values that are used as default for optional arguments. 

# Search the help for a topic:
help.search("t-test")

# If you know the exact name of a function, you can run the examples included in the help page of the function:
example(t.test)

# Download and install a package:
install.packages("car")

# Load previously downloaded and installed packages
library(car)

## ----------------- Variables ---------------------------------

# Variables: objects storing data (also called 'objects' in R)
# Naming variables
#   Names are case-sensitive
#   Names can only begin with letters and periods '.' 
#   Use only the standard alphanumerical characters, the underline 
#'_' and the period '.' characters 

# Expression
#   Two plus the natural logarithm of 3
2 + log(3)     # 2+log(3) 

# Assignment
var1 = 3.87    # Let the variable named var1 be equal to 3.87.
                # You don't have to predefine the variables
# or
var1 <- 3.87

# Printing the object on the screen
var1

# Assignments put in parentheses are printed at once:
(valr = 3.87)

##------------------ Vectors, matrices-------------------------------
# Vectors, matrices: objects capable of storing data similarly to their mathematical equivalents

# Use the function c() to create vectors (c: concatenate)
vector1 = rep(1, 8)
vector2 = c(2,8,4,9,5,6,3)
vector2
vector3 = c(7,3,2,6,1,4,2)
vector3

# Vector: 1,2,3,...,7 
vector4 = 1:7

# Let's examine the following commands!

x = 7
1:x-1
1:(x-1)
x:1

neg = -1:-10
neg

vector5 = c(vector3, 5, 4, 3)
vector5

# Length of a vector:
length(vector4)

# Subsetting (Indexing) vectors:
# Indexes should always be given in squared brackets
# Third element
vector3[5]

# Elements from 1 to 3
vector3[1:3]

# Assigning new values (1,2,3,4,5) to the elements 11-15
vector5[11:15] = c(1, 2, 3, 4, 5)
vector5

# or 
vector5[11:15] = 1:5
vector5

# Create a matrix of 2 rows and 5 columns, filled by columns with numbers 1-10
mat1 = matrix(1:10, ncol = 5,nrow = 2)       # ncol: number of columns
mat1                                        # nrow: number of rows

# Subsetting (Indexing) matrices:
#	1. line 3. element:
mat1[1,3]

# 2. line:
mat1[2,]

# Fill by rows:
mat2 = matrix(1:12, nrow = 3, byrow = TRUE)
mat2

# by row ; 'T' or 'TRUE' ; logical value
# opposite: 'F' or 'FALSE'

# Transpose a matrix:
t(mat2)

# Bind rows and columns: 
rbind(A = 1:4, B = 5:8, C = 9:12)        # rbind: binds rows 
cbind(A = 1:4, B = 5:8, C = 9:12)        # cbind: binds columns

# Text (character) vector:
# Character strings should be put in either double "" or single '' quotes!
sex = c ('M','M','F','M','F','M','F')
sex

# Replicate values, study the results:
rep(c(1,2,3), 3)
rep(1:3, 3)
rep(1:3, 1:3)
rep(c('H','N'), 4)

# Exclude an element:
x[-2]

# Index more than one values:

vector5[1:5]
vector5[c(1, 3, 5)]
elements = c(1, 3, 5)
vector5[elements]

# Exclude multiple elements:
vector5[-c(2, 4)]

# Elements satisfying the condition:
vector5[vector5 > 3]

# When indexing a matrix with only one number, the matching value is found by going through the columns
mat2
mat2[7]

# Is there the number 3 among the elements of the matrix? 
# The answer is given with logical values 
mat2 == 3

# Which elements are equal to 4?
which(mat2 == 4)

# Sum of the elements of the matrix:
sum(mat2)

# How many elements greater than 3 has got the matrix?
mat2 > 3
sum(mat2 > 3)

# Successive difference:
diff(vector3)
vector3

# Name of the objects currently available.
objects()  
rm(list = ls()) # Delete all objects          
objects()           

################### Operations with vectors ####################

# Vectors in expressions: the operations are performed element by element
# Usual operators: +, -, *, /,^ (Exponentiation)

# The results of logical operations are logical values: TRUE (T), FALSE (F)
# logical operators: <, <=, >, >=, == (equal to), != (not equal to)
# ? & (And),  | - (Or),  ! (Not).

# Differences of corresponding vector elements
vector2 = c(2,8,4,9,5,6,3)
vector3 = c(7,3,2,6,1,4,2)
vector2
vector3

vector3 - vector2

# Quotients
vector3/vector2

# Infinity

1/0
log(0)

#################### Missing values ########################

# Notation: NA (Not Available)
# Operations involving NA-s always result in NA-s.

vector6 = c(3, 2, 4, NA, 2, 1, 5)
vector6

# vector6[vector6 == NA] doesn't work. To check whether a value is NA or not, you have to use the is.na() function
is.na(vector6)                                       

#check
vector6-vector3
sum(vector6)
sum(vector6[-4])
sum(vector6[!is.na(vector6)])  # Sum up all the non-missing values of vector6
sum(vector6, na.rm = T)        # Same result: Remove the missing values 

#Calculating the arithmetic mean:
mean(vector6)

mean(vector6, na.rm = T)


## ------------------- Data frames ------------------------------------
# A data frame is  tightly coupled collections of variables.
#	Data frames are matrix-like structures, in which the columns can be of different types but have equal length

# Each observation is recorded in rows, the columns are the measured variables
# You can name the rows if you want

# Creating data frames:
vector4 = 1:7
sex = c('M','M','F','M','M','F','M')
mytable = data.frame(vector2, vector3, vector4, sex)
mytable

# Viewing the data frame:
fix(mytable)

# Close the window before moving on!

# Indexing data frames 
# Data frame indexing is similar to matrix indexing
# mytable[row(s), column(s)]

mytable[1, 2] # First row, second column

mytable[1,] # First row

mytable[c(1, 3, 5),]  # 1., 3., 5. rows

mytable[,4] # Fourth column
# or
mytable[,"sex"] # You can use the column names for indexing
# or
mytable$sex

# Indexing multiple columns:
mytable[, c("vector2", "sex")]

# Subsetting data based on a condition:
# Males only
mytable_M = mytable[mytable$sex == 'M',]
mytable_M

# Males only, where vector2 is greater than 4
mytable_M12 = mytable[mytable$sex == "M" & mytable$vector2 > 4,]


## ------------------- Lists ------------------------------------
#	A list is an ordered array of objects (components).
#	Components can be any type of objects including other lists.
#	A lot of functions return lists as a result.

# Creating a list
list1 = list(
      name = "Robert",
       age = 35,
  children = c("Kate","Erwin"),
   vehicle = list(
     manufacturer = "Toyota",
             year = 2010)
)
list1

# Indexing lists
# Similar to vector indexing, but here you have to use two square brackets [[]] instead of one.
list1[[1]]
list1[[2]]

# If the components of the list are named, you can use the listname$componentname format
# Examining the structure of the list 

str(list1)
list1$name
list1$age

# You can index the elements of the components too
list1[[3]][2]

# Or
list1$children[2]

list1[[4]][[1]]
#Or
list1$vehicle$manufacturer

## ----------- Figuring out object types --------------------------
# If you are not sure about the type of a certain object, use the class() 
# funtion to figure it out
class(vector4)
class(sex)
class(mytable)
class(list1)
class(list1[[2]])

## -------------- Getting data into R -----------------------------

## The WORKING DIRECTORY
# is a directory (folder) on a computer or on an external data storage device, including clouds where R search-es, saves and loads files

# The first task is to set the working directory to where all the data and script files are stored. For example, if data are in the /home/USER/stat folder for the user whose computer user name is USER on a computer with a Linux system, type:
# /home/USER/stat
# or
# ~/stat
# The tilde (~) refers to the user's home directory, and this notation allows users working on multiple computers using Linux systems with different user names. However, this does not work when external drives are attached to the system, since external drives are mounted on the system under the /media/USER folder, and this can not be replaced with a generic string; e.g. on an USB drive labelled ?MYDRIVE? attached to a computer with a Linux system, type:
# >setwd("/media/USER/MYDRIVE/stats")
# On a MS-Windows system, folders are organised in a different way, e.g. to set the folder D:\\mydirectory\stats as a working directory for R type:
# >setwd("D:/mydirectory/stats")
# Note that the directory separator in the R-language using MS-Windows  operating systems (but NOT others) can either be "/" or "\\" (but NOT "\").


# Note that the directory separator on MS-Windowscan either be "/" or "\\" (but NOT "\").
#Please type here your directory
mydirectory = "D:/mydirectory/"
setwd(mydirectory) 

# Copy "FICHYP.csv" to "mydirectory"!
# Importing "FICHYP.csv" to R
mydata = read.table(
  file = "FICHYP.csv",    #the name of the file
  sep = ";",              #the field separator character
  dec = ".",              #the character used in the file for decimal points
  header = TRUE,          #a logical value indicating whether the file 
                          #contains the names of the variables in the first line
  na.string = c("null"))  #a character vector of strings which are to be 
#interpreted as NA values

?read.table

# Viewing table
fix(mydata)

## Head and tail of the table
head(mydata)
tail(mydata)

# Structure of the table 
str(mydata)

# Summary of the whole table
summary(mydata)

# Subsetting data

females = subset(mydata, SEX == "F")
#The sema result with indexing
females = mydata[mydata$SEX == "F",]

# Sorting
mydata.sorted = mydata[sort.list(mydata$RING),]
fix(mydata.sorted)

# Saving to file

## ----Saving the data frame as a text file--------------------------------
write.table(mydata,                     #data frame to be saved
            file      = 'FICHYP1.csv',  #output file name
            sep       = ';',            #the field separator string in the output 
            dec       = '.',            #the string to use for decimal points
            quote     = FALSE,          #a logical value. If TRUE, any character 
                                        #or factor columns will be surrounded by 
                                        #double quotes in the output file
            row.names = FALSE)          #logical value indicating whether the 
                                        #row names of the data frame are to be written 
